#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include <stdlib.h>
#include <map>
#include <iostream>
#include <algorithm>
#include <string>
#include <cctype>
#include <tchar.h>

#include "main.h"
#include "preview.h"
#include "convert.h"
#include "resource.h"

#pragma warning(disable:4996)

using namespace Gdiplus;

const char* APP_NAME = "PopstationMD Free GUI V7.1b";

const int BUFFER_SIZE = 1024;
const int DISC_ID_HEADERS_COUNT = 12;
const char DISC_ID_HEADERS[DISC_ID_HEADERS_COUNT][5] =
{
	"SCUS",
	"SLUS",
	"SLES",
	"SCES",
	"SCED",
	"SLPS",
	"SLPM",
	"SCPS",
	"SLED",
	"SLPS",
	"SIPS",
	"ESPM"
};

const int PARAM_HEADERS_NUM = 4;
const char PARAM_HEADERS[PARAM_HEADERS_NUM][10] =
{
	"num=",
	"level=",
	"icon=",
	"output="
};
const size_t PARAM_LENGTH[PARAM_HEADERS_NUM] =
{
	2,
	6,
	MAX_PATH,
	MAX_PATH
};

LRESULT CALLBACK DlgProc(HWND hWnd, UINT uiMsg, WPARAM wParam, LPARAM lParam);
void OnInit(HWND hWnd);
void OnDiscNumChange(HWND hWnd);
void SetDiscNum(HWND, int);
void OnDropFiles(HWND hWnd, HDROP hDrop);
void OnGetId(HWND hWnd);
void OnPreview(HWND hWnd);
void OnExecute(HWND hWnd);
void SetDropItem(HWND hWnd, char *strFilePath);
void SetDropDir(HWND hWnd, char *bDirPath);
void GetDiscID(char *strFileName, char *strDiscId);
void MakeDiscId(char *strBaseDiscId, long num, char *strDiscId);
void ButtonEnable(HWND hWnd, BOOL bEnable);
void GetParameter(char*, char[PARAM_HEADERS_NUM][MAX_PATH]);
size_t GetParameter(char*, size_t, char*, size_t, size_t);

// O[oϐ
HWND g_hMainWnd;
int g_iImageCount;
int g_iDiscNum;
char g_strTitle[MAX_PATH - 1];

HINSTANCE g_hInst = NULL;
int g_main_wnd_left = 0;
int g_main_wnd_top = 0;
int g_main_wnd_right = 0;
int g_main_wnd_bottom = 0;

BOOL g_bConvert = FALSE;

char g_exec_dir[MAX_PATH]; 
char g_output_dir[MAX_PATH];

GdiplusStartupInput gdiSI;
ULONG_PTR           gdiToken;

//-------------------------------------------------------------------------------------------------
//      C֐
//-------------------------------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	HANDLE hExeCheckMutex, hPrevMutex;

	hPrevMutex = OpenMutex(MUTEX_ALL_ACCESS,FALSE,_T(APP_NAME));
	if(hPrevMutex) {
		::CloseHandle(hPrevMutex);
		return 0; 
	}
	hExeCheckMutex = ::CreateMutex(FALSE,0,APP_NAME);

	g_hInst = hInstance;
	setlocale(LC_ALL,"");

	GdiplusStartup(&gdiToken, &gdiSI, NULL);

	// _CAO\
 	DialogBox(NULL, MAKEINTRESOURCE(IDD_POPS_MD_GUI), NULL, (DLGPROC)DlgProc);

    GdiplusShutdown(gdiToken);

	ReleaseMutex(hExeCheckMutex);

	return 0;
}

//-------------------------------------------------------------------------------------------------
//      R[obN֐
//-------------------------------------------------------------------------------------------------
LRESULT CALLBACK DlgProc(HWND hWnd, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uiMsg) {
	case WM_CLOSE:
		if (g_bConvert == FALSE) {
			ClosePreview();
			CloseConvert();
			PostQuitMessage(0); // WM_QUITbZ[W𔭍s
		} else {
			return FALSE;
		}
        break;
	case WM_DROPFILES:
		OnDropFiles(hWnd, (HDROP)wParam);
		return TRUE;
	case WM_INITDIALOG:
		OnInit(hWnd);
		return TRUE;
	case WM_SYSCOMMAND:
		switch(wParam) {
		case SC_MINIMIZE:
			PostMessage(hWnd, WM_MINIMIZE_AFTER, 0, 0);
			break;
		case SC_RESTORE:
			PostMessage(hWnd, WM_RESTORE_AFTER, 0, 0);
			break;
		}
		break;
	case WM_CONVERT_END:
		g_bConvert = FALSE;
		break;
	case WM_MINIMIZE_AFTER:
		ShowPreviewWnd(0);
		ShowConvertWnd(0);
		break;
	case WM_RESTORE_AFTER:
		ShowPreviewWnd(1);
		ShowConvertWnd(1);
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam)) {
		case IDOK:
			return TRUE;
		case IDCANCEL:
			return TRUE;
		case IDC_DISC_NUM:
			if (HIWORD(wParam) == CBN_SELCHANGE)
			{
				OnDiscNumChange(hWnd);
			}
			return TRUE;
		case IDC_GETID:
			OnGetId(hWnd);
			return TRUE;
		case IDC_PREVIEW:
			OnPreview(hWnd);
			return TRUE;
		case IDC_EXECUTE:
			OnExecute(hWnd);
			return TRUE;
		}
		break;
	default:
		break;
	}
	return FALSE;
}

//-------------------------------------------------------------------------------------------------
//      EBhE֐
//-------------------------------------------------------------------------------------------------
void OnInit(HWND hWnd)
{
	// Np[^̎擾
	char szParams[PARAM_HEADERS_NUM][MAX_PATH];
	GetParameter(GetCommandLine(), szParams);

	// O[oϐ̏
	g_hMainWnd = hWnd;
	g_iDiscNum = 1;
	g_iImageCount = 0;
	memset(g_strTitle, 0, MAX_PATH - 1);

	// ACR̐ݒ
	HICON hIconSm;
	hIconSm = LoadIcon(g_hInst, MAKEINTRESOURCE(IDI_ICON));
	SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM)hIconSm);

	// fBXNXg̐ݒ
	HWND hDropDown;
	hDropDown = GetDlgItem(hWnd, IDC_DISC_NUM);
	SendMessage(hDropDown, CB_ADDSTRING, 1, (LPARAM)"1");
	SendMessage(hDropDown, CB_ADDSTRING, 2, (LPARAM)"2");
	SendMessage(hDropDown, CB_ADDSTRING, 3, (LPARAM)"3");
	SendMessage(hDropDown, CB_ADDSTRING, 4, (LPARAM)"4");
	SendMessage(hDropDown, CB_ADDSTRING, 5, (LPARAM)"5");
	if (strlen(szParams[0]) > 0 && atoi(szParams[0]) > 0 && atoi(szParams[0]) <= 6) {
		g_iDiscNum = atoi(szParams[0]);
		SendMessage(hDropDown, CB_SETCURSEL, g_iDiscNum - 1, 0);
		SetDiscNum(hWnd, g_iDiscNum);
	} else {
		SendMessage(hDropDown, CB_SETCURSEL, 0, 0);
	}

	// fBXNID̍ő͕
	HWND hEdit;
	hEdit =  GetDlgItem(hWnd, IDC_DISCID1);
	SendMessage(hEdit, EM_SETLIMITTEXT, 9, 0);
	hEdit =  GetDlgItem(hWnd, IDC_DISCID2);
	SendMessage(hEdit, EM_SETLIMITTEXT, 9, 0);
	hEdit =  GetDlgItem(hWnd, IDC_DISCID3);
	SendMessage(hEdit, EM_SETLIMITTEXT, 9, 0);
	hEdit =  GetDlgItem(hWnd, IDC_DISCID4);
	SendMessage(hEdit, EM_SETLIMITTEXT, 9, 0);
	hEdit =  GetDlgItem(hWnd, IDC_DISCID5);
	SendMessage(hEdit, EM_SETLIMITTEXT, 9, 0);

	// XsRg[̐ݒ
	HWND hSpin;
	size_t nLen = 0;
	int nWorkLevel = 9;
	char szWorkLevel[2] = "9";
	int nDefaultLevel[5] = {9, 9, 9, 9, 9};
	char szDefaultLevel[5][2] = {"9", "9", "9", "9", "9"};

	// Np[^珉l擾
	nLen = strlen(szParams[1]);
	if (nLen > 0) {
		for (size_t i = 0; i < 5; i++) {
			if (i < strlen(szParams[1]) && szParams[1][i] >= '0' && szParams[1][i] <= '9') {
				nWorkLevel = szParams[1][i] - '0';
				sprintf(szWorkLevel, "%d", nWorkLevel);
				nDefaultLevel[i] = nWorkLevel;
				strcpy(szDefaultLevel[i], szWorkLevel);
			} else {
				nDefaultLevel[i] = nWorkLevel;
				strcpy(szDefaultLevel[i], szWorkLevel);
			}
		}
	}

	// Pڂ̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN1);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 0, 9);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, nDefaultLevel[0]);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_LEVEL1);
	// ftHgݒ
	SetWindowText(hEdit, szDefaultLevel[0]);
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// Qڂ̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN2);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 0, 9);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, nDefaultLevel[1]);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_LEVEL2);
	// ftHgݒ
	SetWindowText(hEdit, szDefaultLevel[1]);
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// Rڂ̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN3);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 0, 9);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, nDefaultLevel[2]);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_LEVEL3);
	// ftHgݒ
	SetWindowText(hEdit, szDefaultLevel[2]);
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// Sڂ̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN4);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 0, 9);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, nDefaultLevel[3]);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_LEVEL4);
	// ftHgݒ
	SetWindowText(hEdit, szDefaultLevel[3]);
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// Tڂ̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN5);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 0, 9);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, nDefaultLevel[4]);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_LEVEL5);
	// ftHgݒ
	SetWindowText(hEdit, szDefaultLevel[4]);
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// x̐ݒ
	hSpin = GetDlgItem(hWnd, IDC_PARENTAL_SPIN);
	// l͈̔͂ݒ肷
	SendMessage(hSpin, UDM_SETRANGE32, 1, 11);
	// lݒ
	SendMessage(hSpin, UDM_SETPOS32, 0, 1);
	// GfBbg{bNX̃nh擾
	hEdit = GetDlgItem(hWnd, IDC_PARENTAL);
	// ftHgݒ
	SetWindowText(hEdit, "1");
	// GfBbg{bNXƊ֘AÂ
	SendMessage(hSpin, UDM_SETBUDDY, (WPARAM)hEdit, 0);

	// [WR[h̐ݒ
	hEdit = GetDlgItem(hWnd, IDC_REGION);
	// ftHgݒ
	SetWindowText(hEdit, "8000");
	SendMessage(hEdit, EM_SETLIMITTEXT, 4, 0);

	// ACRi[tH_
	if (strlen(szParams[2]) > 0) {
		SetDlgItemText(hWnd, IDC_ICON_DIR, szParams[2]);
	}

	// o͐̐ݒ
	if (strlen(szParams[3]) > 0) {
		strncpy(g_output_dir, szParams[3], MAX_PATH);
		if (g_output_dir[strlen(g_output_dir) - 1] != '\\') {
			strncat(g_output_dir, "\\", MAX_PATH);
		}
	} else {
		strncpy(g_output_dir, g_exec_dir, MAX_PATH);
	}

	// EBhEőOʂɕ\
	SetWindowPos(hWnd, HWND_TOPMOST , 0, 0, 0, 0, SWP_NOSIZE);

	g_bConvert = FALSE;
}

//-------------------------------------------------------------------------------------------------
//      fBXNύX֐
//-------------------------------------------------------------------------------------------------
void OnDiscNumChange(HWND hWnd)
{
	char strDiscNum[2];

	GetDlgItemText(hWnd, IDC_DISC_NUM, strDiscNum, 2);
	sscanf(strDiscNum, "%d", &g_iDiscNum);

	SetDiscNum(hWnd, g_iDiscNum);
}

void SetDiscNum(HWND hWnd, int nDiscNum)
{
	HWND hImage;
	HWND hDiscId;
	HWND hLevel;
	HWND hSpin;

	// 2
	hImage = GetDlgItem(hWnd, IDC_IMAGE2);
	hDiscId = GetDlgItem(hWnd, IDC_DISCID2);
	hLevel = GetDlgItem(hWnd, IDC_LEVEL2);
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN2);
	if (g_iDiscNum >= 2) {
		EnableWindow(hImage, true);
		EnableWindow(hDiscId, true);
		EnableWindow(hLevel, true);
		EnableWindow(hSpin, true);
	} else {
		EnableWindow(hImage, false);
		EnableWindow(hDiscId, false);
		EnableWindow(hLevel, false);
		EnableWindow(hSpin, false);
	}

	// 3
	hImage = GetDlgItem(hWnd, IDC_IMAGE3);
	hDiscId = GetDlgItem(hWnd, IDC_DISCID3);
	hLevel = GetDlgItem(hWnd, IDC_LEVEL3);
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN3);
	if (g_iDiscNum >= 3) {
		EnableWindow(hImage, true);
		EnableWindow(hDiscId, true);
		EnableWindow(hLevel, true);
		EnableWindow(hSpin, true);
	} else {
		EnableWindow(hImage, false);
		EnableWindow(hDiscId, false);
		EnableWindow(hLevel, false);
		EnableWindow(hSpin, false);
	}

	// 4
	hImage = GetDlgItem(hWnd, IDC_IMAGE4);
	hDiscId = GetDlgItem(hWnd, IDC_DISCID4);
	hLevel = GetDlgItem(hWnd, IDC_LEVEL4);
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN4);
	if (g_iDiscNum >= 4) {
		EnableWindow(hImage, true);
		EnableWindow(hDiscId, true);
		EnableWindow(hLevel, true);
		EnableWindow(hSpin, true);
	} else {
		EnableWindow(hImage, false);
		EnableWindow(hDiscId, false);
		EnableWindow(hLevel, false);
		EnableWindow(hSpin, false);
	}

	// 5
	hImage = GetDlgItem(hWnd, IDC_IMAGE5);
	hDiscId = GetDlgItem(hWnd, IDC_DISCID5);
	hLevel = GetDlgItem(hWnd, IDC_LEVEL5);
	hSpin = GetDlgItem(hWnd, IDC_LEVEL_SPIN5);
	if (g_iDiscNum >= 5) {
		EnableWindow(hImage, true);
		EnableWindow(hDiscId, true);
		EnableWindow(hLevel, true);
		EnableWindow(hSpin, true);
	} else {
		EnableWindow(hImage, false);
		EnableWindow(hDiscId, false);
		EnableWindow(hLevel, false);
		EnableWindow(hSpin, false);
	}
}

//-------------------------------------------------------------------------------------------------
//      hbvt@C֐
//-------------------------------------------------------------------------------------------------
void OnDropFiles(HWND hWnd, HDROP hDrop)
{
	// hbvꂽt@C̐擾
	long count;
	count = DragQueryFile(hDrop, -1, NULL, NULL);

	long i;
	long j;
	bool imgEnd;
	char strFilePath[MAX_PATH - 1];
	char strTitle[128];
	long nIndex;
	long nLen;

	imgEnd = false;

	for (i = 0; i < count; i++) {
		// et@C擾
		DragQueryFile(hDrop, i, strFilePath, MAX_PATH);

		if ((GetFileAttributes(strFilePath) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) {
			SetDropDir(hWnd, strFilePath);
			if (g_iImageCount >= g_iDiscNum) {
				nIndex = 0;
				nLen = lstrlen(strFilePath);
				for (j = nLen - 1; j >= 0; j--) {
					if (j == nLen - 1 && strFilePath[j] == '\\' ) {
						continue;
					}
					if (strFilePath[j] == '\\') {
						if (j > 0) {
							if (strFilePath[j - 1] >= (char)0x81 && strFilePath[j - 1] <= (char)0x9f) {
								continue;
							}
							if (strFilePath[j - 1] >= (char)0xe0 && strFilePath[j - 1] <= (char)0xfc) {
								continue;
							}
						}
						nIndex = j + 2;
						break;
					}
				}
				if (strFilePath[nLen - 1] == '\\' ) {
					strFilePath[nLen - 1] = 0;
					lstrcpy(strTitle, &strFilePath[nIndex - 1]);
				} else {
					lstrcpy(strTitle, &strFilePath[nIndex - 1]);
				}
				SetDlgItemText(hWnd, IDC_TITLE, strTitle);
			}
		} else if (imgEnd == false) { 
			SetDropItem(hWnd, strFilePath);

			if (g_iImageCount >= g_iDiscNum) {
				SetDlgItemText(hWnd, IDC_TITLE, g_strTitle);
			}
		}

		if (g_iImageCount >= g_iDiscNum) {
			g_iImageCount = 0;
			g_strTitle[0] = 0;
			imgEnd = true;
		}
	}
}

//-------------------------------------------------------------------------------------------------
//      hbvACe֐
//-------------------------------------------------------------------------------------------------
void SetDropItem(HWND hWnd, char *strFilePath)
{
	long i;
	long len;
	char strDrive[MAX_PATH - 1];
	char strDir[MAX_PATH - 1];
	char strFileName[MAX_PATH - 1];
	char strExt[MAX_PATH - 1];
	char strTitle[MAX_PATH - 1];

	// t@CpX𕪊
	_splitpath(strFilePath, strDrive, strDir, strFileName, strExt);

	if (lstrcmpi(strExt, ".ISO") != 0 && lstrcmpi(strExt, ".IMG") != 0  && lstrcmpi(strExt, ".BIN") != 0) {
		return;
	}

	// GfBbg{bNXɃt@Cǉ
	if (g_iImageCount == 0) {
		SetDlgItemText(hWnd, IDC_IMAGE1, strFilePath);
	} else if (g_iImageCount == 1) {
		SetDlgItemText(hWnd, IDC_IMAGE2, strFilePath);
	} else if (g_iImageCount == 2) {
		SetDlgItemText(hWnd, IDC_IMAGE3, strFilePath);
	} else if (g_iImageCount == 3) {
		SetDlgItemText(hWnd, IDC_IMAGE4, strFilePath);
	} else if (g_iImageCount == 4) {
		SetDlgItemText(hWnd, IDC_IMAGE5, strFilePath);
	}

	lstrcpy(strTitle, strFileName);
	if (lstrlen(g_strTitle) <= 0) {
		lstrcpy(g_strTitle, strTitle);
	} else {
		len = lstrlen(strTitle);
		for (i = 0; i < len; i++) {
			if (g_strTitle[i] != strTitle[i]) {
				g_strTitle[i] = 0;
			}
		}
	}

	// C[Wt@CCNg
	g_iImageCount++;
}

//-------------------------------------------------------------------------------------------------
//      hbvtH_֐
//-------------------------------------------------------------------------------------------------
void SetDropDir(HWND hWnd, char *strDirPath) {
	char strFindDir[MAX_PATH];
	char strFindFile[MAX_PATH];
	char strFilePath[MAX_PATH];
	HANDLE hFindHandle;
	WIN32_FIND_DATA strFindData;
	bool bIcon;
	std::string file;
	std::map<std::string, std::string> files;
	std::map<std::string, std::string>::iterator it;

	bIcon = false;

	// pX쐬
	lstrcpy(strFindDir, strDirPath);
	if (strFindDir[lstrlen(strFindDir) - 1] != '\\' ) {
		lstrcat(strFindDir, "\\");
	}
	lstrcpy(strFindFile, strFindDir);
	lstrcat(strFindFile, "*");

	hFindHandle = FindFirstFile(strFindFile, &strFindData);

	if (hFindHandle == INVALID_HANDLE_VALUE) {
		return;
	}

	do {
		if (lstrcmp(strFindData.cFileName, ".") == 0 || lstrcmp(strFindData.cFileName, "..") == 0) {
			continue;
		}
		file = strFindData.cFileName;
		std::transform(file.begin(), file.end(), file.begin(), toupper);
		files.insert(std::map<std::string, std::string>::value_type(file, strFindData.cFileName));
	} while (FindNextFile(hFindHandle, &strFindData) == TRUE);

	FindClose(hFindHandle);

	it = files.begin();
	while( it != files.end() ) {
		lstrcpy(strFilePath, strFindDir);
		lstrcat(strFilePath, (*it).second.c_str());
		if (g_iImageCount < g_iDiscNum) {
			SetDropItem(hWnd, strFilePath);
		}
		if ((GetFileAttributes(strFilePath) & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) {
			SetDropDir(hWnd, strFilePath);
		}
		if (lstrcmpi((*it).second.c_str(), "icon0.png") == 0) {
			bIcon = true;
		} else if (lstrcmpi((*it).second.c_str(), "icon1.pmf") == 0) {
			bIcon = true;
		} else if (lstrcmpi((*it).second.c_str(), "pic0.png") == 0) {
			bIcon = true;
		} else if (lstrcmpi((*it).second.c_str(), "pic1.png") == 0) {
			bIcon = true;
		} else if (lstrcmpi((*it).second.c_str(), "snd0.at3") == 0) {
			bIcon = true;
		} else if (lstrcmpi((*it).second.c_str(), "splash.png") == 0) {
			bIcon = true;
		}
		++it;
	}

	// vf̑S폜
	files.clear();

	if (bIcon == true) {
		SetDlgItemText(hWnd, IDC_ICON_DIR, strDirPath);
	}
}

//-------------------------------------------------------------------------------------------------
//      ID擾{^NbN֐
//-------------------------------------------------------------------------------------------------
void OnGetId(HWND hWnd)
{
	char strFileName[MAX_PATH];
	char strDiscId[10];

	GetDlgItemText(hWnd, IDC_IMAGE1, strFileName, MAX_PATH);
	if (lstrlen(strFileName) <= 0 ) {
		MessageBox(hWnd, "fBXNP̃C[Wt@Cw肵Ă", "fBXNID擾G[", MB_ICONINFORMATION);
		return;
	}
	GetDiscID(strFileName, strDiscId);
	if (lstrlen(strDiscId) == 9 ) {
		SetDlgItemText(hWnd, IDC_DISCID1, strDiscId);
	} else {
		MessageBox(hWnd, "fBXNPID擾ł܂ł", "fBXNID擾G[", MB_ICONINFORMATION);
	}

	if (g_iDiscNum >= 2 ) {
		GetDlgItemText(hWnd, IDC_IMAGE2, strFileName, MAX_PATH);
		if (lstrlen(strFileName) <= 0 ) {
			MessageBox(hWnd, "fBXNQ̃C[Wt@Cw肵Ă", "fBXNID擾G[", MB_ICONINFORMATION);
			return;
		}
		GetDiscID(strFileName, strDiscId);
		if (lstrlen(strDiscId) == 9 ) {
			SetDlgItemText(hWnd, IDC_DISCID2, strDiscId);
		} else {
			MessageBox(hWnd, "fBXNQID擾ł܂ł", "fBXNID擾G[", MB_ICONINFORMATION);
		}
	}

	if (g_iDiscNum >= 3 ) {
		GetDlgItemText(hWnd, IDC_IMAGE3, strFileName, MAX_PATH);
		if (lstrlen(strFileName) <= 0 ) {
			MessageBox(hWnd, "fBXNR̃C[Wt@Cw肵Ă", "fBXNID擾G[", MB_ICONINFORMATION);
			return;
		}
		GetDiscID(strFileName, strDiscId);
		if (lstrlen(strDiscId) == 9 ) {
			SetDlgItemText(hWnd, IDC_DISCID3, strDiscId);
		} else {
			MessageBox(hWnd, "fBXNRID擾ł܂ł", "fBXNID擾G[", MB_ICONINFORMATION);
		}
	}

	if (g_iDiscNum >= 4 ) {
		GetDlgItemText(hWnd, IDC_IMAGE4, strFileName, MAX_PATH);
		if (lstrlen(strFileName) <= 0 ) {
			MessageBox(hWnd, "fBXNS̃C[Wt@Cw肵Ă", "fBXNID擾G[", MB_ICONINFORMATION);
			return;
		}
		GetDiscID(strFileName, strDiscId);
		if (lstrlen(strDiscId) == 9 ) {
			SetDlgItemText(hWnd, IDC_DISCID4, strDiscId);
		} else {
			MessageBox(hWnd, "fBXNSID擾ł܂ł", "fBXNID擾G[", MB_ICONINFORMATION);
		}
	}

	if (g_iDiscNum >= 5 ) {
		GetDlgItemText(hWnd, IDC_IMAGE5, strFileName, MAX_PATH);
		if (lstrlen(strFileName) <= 0 ) {
			MessageBox(hWnd, "fBXNT̃C[Wt@Cw肵Ă", "fBXNID擾G[", MB_ICONINFORMATION);
			return;
		}
		GetDiscID(strFileName, strDiscId);
		if (lstrlen(strDiscId) == 9 ) {
			SetDlgItemText(hWnd, IDC_DISCID5, strDiscId);
		} else {
			MessageBox(hWnd, "fBXNTID擾ł܂ł", "fBXNID擾G[", MB_ICONINFORMATION);
		}
	}
}

//-------------------------------------------------------------------------------------------------
//      fBXNID擾֐
//-------------------------------------------------------------------------------------------------
void GetDiscID(char *strFileName, char *strDiscId)
{
	FILE *hFile;
	char buffer[BUFFER_SIZE];
	long readsize;
	long i;
	long j;
	long k;
	long l;
	bool bUXAFlag;
	bool bHeaderFlag;
	char strID[10];
	char strUXA[4];

	hFile = fopen(strFileName, "rb");
			
	if (hFile != NULL) {
		// C000܂ŃV[N
		if (fseek(hFile, 0xC000, SEEK_SET) != 0){
			fclose(hFile);
			return;
		}

		// 
		k = 0;
		bHeaderFlag = false;
		bUXAFlag = false;
		strID[0] = 0; 
		strUXA[0] = 0;
		for (i = 0; i < 100; i++) {
			// t@Cǂݍ
			readsize = (long)fread(buffer, sizeof(char), BUFFER_SIZE - 1, hFile);
			for (j = 0; j < readsize; j++) {
				if (bUXAFlag == true) {
					if (buffer[j] == '$' || buffer[j] == 0) {
						strID[0] = 0;
						k = 0;
					} else {
						if (bHeaderFlag == true) {
							if (buffer[j] == 0) {
								strID[0] = 0;
								k = 0;
								bHeaderFlag = false;
							} else if (buffer[j] >= '0' && buffer[j] <= '9') {
								strID[k] = buffer[j];
								strID[k + 1] = 0;
								k++;
							}
							if (lstrlen(strID) == 9) {
								break;
							}
						} else {
							if (k == 0 && (buffer[j] == 'S' || buffer[j] == 'E')) {
								strID[k] = buffer[j];
								strID[k + 1] = 0;
								k++;
							} else if (k == 1 && (buffer[j] == 'C' || buffer[j] == 'L' || buffer[j] == 'I' || buffer[j] == 'S')) {
								strID[k] = buffer[j];
								strID[k + 1] = 0;
								k++;
							} else if (k == 2 && (buffer[j] == 'U' || buffer[j] == 'E' || buffer[j] == 'P')) {
								strID[k] = buffer[j];
								strID[k + 1] = 0;
								k++;
							} else if (k == 3 && (buffer[j] == 'S' || buffer[j] == 'D' || buffer[j] == 'M')) {
								strID[k] = buffer[j];
								strID[k + 1] = 0;
								k++;
							} else {
								strID[0] = 0;
								k = 0;
							}
							if (k == 4) {
								for (l = 0; l < DISC_ID_HEADERS_COUNT; l++) {
									if (lstrcmp(strID, DISC_ID_HEADERS[l]) == 0) {
										bHeaderFlag = true;
										break;
									}
								}
								if (bHeaderFlag == false) {
									strID[0] = 0;
									k = 0;
								}
							}
						}
					}
				} else {
					if (k == 0 && buffer[j] == 'U') {
						k++;
					} else if (k == 1 && buffer[j] == 'X') {
						k++;
					} else if (k == 2 && buffer[j] == 'A') {
						k++;
					} else {
						k = 0;
					}
					if (k == 3) {
						bUXAFlag = true;
						k = 0;
					}
				}
			}

			if (lstrlen(strID) == 9) {
				break;
			}
		}

		// t@C
		fclose(hFile);
	}

	lstrcpy(strDiscId, strID);
}

//-------------------------------------------------------------------------------------------------
//      ACRvr[֐
//-------------------------------------------------------------------------------------------------
void OnPreview(HWND hWnd)
{
	char strIconDir[MAX_PATH];
	HWND hCmdPreview;
	RECT rect;

	GetDlgItemText(hWnd, IDC_ICON_DIR, strIconDir, MAX_PATH - 1);
	if (lstrlen(strIconDir) > 0 && strIconDir[lstrlen(strIconDir) - 1] != '\\') {
		lstrcat(strIconDir, "\\");
	}
	SetIconDir(strIconDir);

	GetWindowRect(hWnd, &rect);
	g_main_wnd_left = rect.left;
	g_main_wnd_top = rect.top;
	g_main_wnd_right = rect.right;
	g_main_wnd_bottom = rect.bottom;

	hCmdPreview = GetDlgItem(hWnd, IDC_PREVIEW);

	EnableWindow(hCmdPreview, FALSE);

	// _CAO\
 	DialogBox(NULL, MAKEINTRESOURCE(IDD_ICON_PREVIEW), NULL, (DLGPROC)DlgProcPreview);

	EnableWindow(hCmdPreview, TRUE);
}

bool checkid(char* id)
{
	int i;

	for (i = 0; i < DISC_ID_HEADERS_COUNT; i++)
	{
		if (strncmp(id, DISC_ID_HEADERS[i], 4) == 0)
		break;
	}

	if (i == DISC_ID_HEADERS_COUNT)
	{
		return false;
	}

	for (i = 4; i < 9; i++)
	{
		if (id[i] < '0' || id[i] > '9')
		{
			return false;
		}
	}

	return true;
}

//-------------------------------------------------------------------------------------------------
//      ϊs֐
//-------------------------------------------------------------------------------------------------
void OnExecute(HWND hWnd)
{
	char strBaseDiscId[10];
	char buffer[BUFFER_SIZE];

	char title[0x80];
	char inputs[MAX_DISC_NUM][MAX_PATH];
	char discids[MAX_DISC_NUM][10];
	char levels[MAX_DISC_NUM][2];
	char parental[3];
	char region[5];
	char icon_dir[MAX_PATH];

	RECT rect;

	ClosePreview();

	GetDlgItemText(hWnd, IDC_TITLE, buffer, BUFFER_SIZE);
	if (lstrlen(buffer) <= 0) {
		MessageBox(hWnd, "^Cgw肵Ă", "sG[", MB_ICONINFORMATION);
		return;
	}

	GetDlgItemText(hWnd, IDC_IMAGE1, buffer, BUFFER_SIZE);
	if (lstrlen(buffer) <= 0) {
		MessageBox(hWnd, "fBXNP̃t@Cw肵Ă", "sG[", MB_ICONINFORMATION);
		return;
	}

	if (g_iDiscNum >= 2) {
		GetDlgItemText(hWnd, IDC_IMAGE2, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MessageBox(hWnd, "fBXNQ̃t@Cw肵Ă", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 3) {
		GetDlgItemText(hWnd, IDC_IMAGE3, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MessageBox(hWnd, "fBXNR̃t@Cw肵Ă", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 4) {
		GetDlgItemText(hWnd, IDC_IMAGE4, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MessageBox(hWnd, "fBXNS̃t@Cw肵Ă", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 5) {
		GetDlgItemText(hWnd, IDC_IMAGE5, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MessageBox(hWnd, "fBXNT̃t@Cw肵Ă", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	GetDlgItemText(hWnd, IDC_DISCID1, buffer, BUFFER_SIZE);
	if (lstrlen(buffer) == 9) {
		if (checkid(buffer) == false) {
			MessageBox(hWnd, "Pڂ̃fBXNIDsł", "sG[", MB_ICONINFORMATION);
			return;
		}
	} else {
		MessageBox(hWnd, "X̃fBXNID͂Ă", "sG[", MB_ICONINFORMATION);
		return;
	}

	if (g_iDiscNum >= 2) {
		GetDlgItemText(hWnd, IDC_DISCID2, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) > 0 && checkid(buffer) == false) {
			MessageBox(hWnd, "Qڂ̃fBXNIDsł", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 3) {
		GetDlgItemText(hWnd, IDC_DISCID3, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) > 0 && checkid(buffer) == false) {
			MessageBox(hWnd, "Rڂ̃fBXNIDsł", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 4) {
		GetDlgItemText(hWnd, IDC_DISCID4, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) > 0 && checkid(buffer) == false) {
			MessageBox(hWnd, "Sڂ̃fBXNIDsł", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	if (g_iDiscNum >= 5) {
		GetDlgItemText(hWnd, IDC_DISCID5, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) > 0 && checkid(buffer) == false) {
			MessageBox(hWnd, "Tڂ̃fBXNIDsł", "sG[", MB_ICONINFORMATION);
			return;
		}
	}

	// ^Cg擾
	GetDlgItemText(hWnd, IDC_TITLE, buffer, BUFFER_SIZE);
	memset(title, 0, 0x80);
	strncpy(title, buffer, 0x80);

	// fBXNP̏擾
	GetDlgItemText(hWnd, IDC_IMAGE1, buffer, BUFFER_SIZE);
	memset(inputs[0], 0, MAX_PATH);
	strncpy(inputs[0], buffer, MAX_PATH);
	GetDlgItemText(hWnd, IDC_DISCID1, buffer, BUFFER_SIZE);
	memset(discids[0], 0, 10);
	strncpy(discids[0], buffer, 10);
	GetDlgItemText(hWnd, IDC_LEVEL1, buffer, BUFFER_SIZE);
	memset(levels[0], 0, 2);
	strncpy(levels[0], buffer, 2);

	// fBXNQ̏擾
	if (g_iDiscNum >= 2) {
		GetDlgItemText(hWnd, IDC_IMAGE2, buffer, BUFFER_SIZE);
		memset(inputs[1], 0, MAX_PATH);
		strncpy(inputs[1], buffer, MAX_PATH);
		GetDlgItemText(hWnd, IDC_DISCID2, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MakeDiscId(strBaseDiscId, 2, buffer);
		}
		memset(discids[1], 0, 10);
		strncpy(discids[1], buffer, 10);
		GetDlgItemText(hWnd, IDC_LEVEL2, buffer, BUFFER_SIZE);
		memset(levels[1], 0, 2);
		strncpy(levels[1], buffer, 2);
	}

	// fBXNȐ擾
	if (g_iDiscNum >= 3) {
		GetDlgItemText(hWnd, IDC_IMAGE3, buffer, BUFFER_SIZE);
		memset(inputs[2], 0, MAX_PATH);
		strncpy(inputs[2], buffer, MAX_PATH);
		GetDlgItemText(hWnd, IDC_DISCID3, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MakeDiscId(strBaseDiscId, 3, buffer);
		}
		memset(discids[2], 0, 10);
		strncpy(discids[2], buffer, 10);
		GetDlgItemText(hWnd, IDC_LEVEL3, buffer, BUFFER_SIZE);
		memset(levels[2], 0, 2);
		strncpy(levels[2], buffer, 2);
	}

	// fBXNS̏擾
	if (g_iDiscNum >= 4) {
		GetDlgItemText(hWnd, IDC_IMAGE4, buffer, BUFFER_SIZE);
		memset(inputs[3], 0, MAX_PATH);
		strncpy(inputs[3], buffer, MAX_PATH);
		GetDlgItemText(hWnd, IDC_DISCID4, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MakeDiscId(strBaseDiscId, 4, buffer);
		}
		memset(discids[3], 0, 10);
		strncpy(discids[3], buffer, 10);
		GetDlgItemText(hWnd, IDC_LEVEL4, buffer, BUFFER_SIZE);
		memset(levels[3], 0, 2);
		strncpy(levels[3], buffer, 2);
	}

	// fBXNT̏擾
	if (g_iDiscNum >= 5) {
		GetDlgItemText(hWnd, IDC_IMAGE5, buffer, BUFFER_SIZE);
		memset(inputs[4], 0, MAX_PATH);
		strncpy(inputs[4], buffer, MAX_PATH);
		GetDlgItemText(hWnd, IDC_DISCID5, buffer, BUFFER_SIZE);
		if (lstrlen(buffer) <= 0) {
			MakeDiscId(strBaseDiscId, 5, buffer);
		}
		memset(discids[4], 0, 10);
		strncpy(discids[4], buffer, 10);
		GetDlgItemText(hWnd, IDC_LEVEL5, buffer, BUFFER_SIZE);
		memset(levels[4], 0, 2);
		strncpy(levels[4], buffer, 2);
	}

	// ACRi[tH_擾
	GetDlgItemText(hWnd, IDC_ICON_DIR, buffer, BUFFER_SIZE);
	memset(icon_dir, 0, MAX_PATH);
	if (lstrlen(buffer) > 0) {
		if (buffer[lstrlen(buffer) - 1] != '\\') {
			lstrcat(buffer, "\\");
		}
		strncpy(icon_dir, buffer, MAX_PATH);
	}

	// x̎擾
	GetDlgItemText(hWnd, IDC_PARENTAL, buffer, BUFFER_SIZE);
	memset(parental, 0, 3);
	strncpy(parental, buffer, 3);

	// [WR[h̎擾
	GetDlgItemText(hWnd, IDC_REGION, buffer, BUFFER_SIZE);
	memset(region, 0, 5);
	strncpy(region, buffer, 5);

	SetConvertData(title, g_iDiscNum, inputs, discids, levels, icon_dir, parental, region);

	GetWindowRect(hWnd, &rect);
	g_main_wnd_left = rect.left;
	g_main_wnd_top = rect.top;
	g_main_wnd_right = rect.right;
	g_main_wnd_bottom = rect.bottom;

	SetWindowPos(hWnd, HWND_NOTOPMOST , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	ButtonEnable(hWnd, FALSE);
	g_bConvert = TRUE;

	// _CAO\
 	DialogBox(NULL, MAKEINTRESOURCE(IDD_CONVERT), NULL, (DLGPROC)DlgProcConvert);

	g_bConvert = FALSE;
	ButtonEnable(hWnd, TRUE);
	SetWindowPos(hWnd, HWND_TOPMOST , 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
}

void ButtonEnable(HWND hWnd, BOOL bEnable)
{
	HWND hCmdGetID;
	HWND hCmdExecute;
	HWND hCmdPreview;

	hCmdGetID = GetDlgItem(hWnd, IDC_GETID);
	hCmdExecute = GetDlgItem(hWnd, IDC_EXECUTE);
	hCmdPreview = GetDlgItem(hWnd, IDC_PREVIEW);

	EnableWindow(hCmdGetID, bEnable);
	EnableWindow(hCmdExecute, bEnable);
	EnableWindow(hCmdPreview, bEnable);
}

void MakeDiscId(char *strBaseDiscId, long num, char *strDiscId)
{
	long i;	
	long code;
	
	for (i = 0; i < 4; i++) {
		strDiscId[i] = strBaseDiscId[i];
	}
	strDiscId[i] = 0;
	
	try {
		sscanf(&strBaseDiscId[4], "%d", &code);
		code += num - 1;
		sprintf(&strDiscId[4], "%05d", code);
	} catch(int e) {
		printf("%d", e);
	}
}

void GetParameter(char* szCmdLine, char szParams[PARAM_HEADERS_NUM][MAX_PATH])
{
	char szAppPath[MAX_PATH];
	size_t nLen = strlen(szCmdLine);
	size_t nStartIndex = 0;
	int paramCount = 0;

	for (int i = 0; i < PARAM_HEADERS_NUM; i++) {
		memset(szParams[i], 0, PARAM_LENGTH[i]);
	}

	nStartIndex = GetParameter(szCmdLine, nLen, szAppPath, MAX_PATH, nStartIndex);
	strncpy(g_exec_dir, szAppPath, strlen(szAppPath) - strlen(strrchr(szAppPath, '\\')) + 1);

	for (; nStartIndex < nLen; nStartIndex++) {
		for (int i = 0; i < PARAM_HEADERS_NUM; i++) {
			if (strncmp(&szCmdLine[nStartIndex], PARAM_HEADERS[i], strlen(PARAM_HEADERS[i])) == 0) {
				nStartIndex = GetParameter(szCmdLine, nLen, szParams[i], PARAM_LENGTH[i], nStartIndex + strlen(PARAM_HEADERS[i]));
				paramCount++;
			}
		}

		if (paramCount == PARAM_HEADERS_NUM) {
			break;
		}
	}
}

size_t GetParameter(char* szCmdLine, size_t nCmdLineLen, char* szParam, size_t nSize, size_t nStartIndex)
{
	char szWork[MAX_PATH];
	bool bQuota = false;
	size_t nWorkIndex = 0;
	size_t nEndIndex = 0;

	memset(szWork, 0, MAX_PATH);

	for (nEndIndex = nStartIndex; nEndIndex < nCmdLineLen; nEndIndex++) {
		if (bQuota == false && szCmdLine[nEndIndex] == '"') {
			bQuota = true;
			continue;
		}

		if (bQuota == true && szCmdLine[nEndIndex] == '"') {
			if (nEndIndex == nCmdLineLen - 1 || szCmdLine[nEndIndex + 1] != '"') {
				bQuota = false;
				break;
			}
		}

		if (bQuota == false && szCmdLine[nEndIndex] == ' ') {
			break;
		}

		if (nWorkIndex < MAX_PATH) {
			szWork[nWorkIndex++] = szCmdLine[nEndIndex];
		}
	}

	if (nWorkIndex < nSize) {
		strncpy(szParam, szWork, nSize);
	}

	return nEndIndex;
}
